﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectStormtrooper.CollectionGenericObjects;
/// <summary>
/// Параметрический набор объектов
/// </summary>
/// <typeparam name="T">Параметр: ограничение - ссылочный тип</typeparam>
public class ListGenericObjects<T> : ICollectionGenericObjects<T>
    where T : class
{
    /// <summary>
    /// Список объектов, которые храним
    /// </summary>
    private readonly List<T?> _collection;
    /// <summary>
    /// Максимально допустимое значение числа объектов в списке
    /// </summary>
    private int _maxCount;
    public int Count => _collection.Count;
    public int SetMaxCount { set { if (value > 0) { _maxCount = value; } } }
    /// <summary>
    /// Конструктор
    /// </summary>
    public ListGenericObjects()
    {
        _collection = new();
    }

    public T? Get(int position)
    {
        if (position>=0 && position < _collection.Count)
        {
            return _collection[position];
        }
        return null;
    }

    public int Insert(T obj)
    {
        // TODO проверка, что не превышено максимальное количество элементов
        // TODO вставка в конец набора
        if (_collection.Count <= _maxCount)
        {
            _collection.Add(obj);
            return _collection.Count;
        }
        return -1;
    }

    public int Insert(T obj, int position)
    {
        // TODO проверка, что не превышено максимальное количество элементов
        // TODO проверка позиции
        // TODO вставка по позиции
        if (position >= 0 && position < _maxCount && _collection.Count <= _maxCount)
        {
            _collection.Insert(position, obj);
            return position;
        }
        return -1;
    }

    public T? Remove(int position)
    {
        // TODO проверка позиции
        // TODO удаление объекта из списка
        if (position < 0 || position > _maxCount)
        {
            return null;
        }
        T temp = _collection[position];
        _collection.RemoveAt(position);
        return temp;
    }
}
